@@ -1,3 +1,21 @@
+/* Copyright 2001, 2002 b8_bavard, b8_fee_carabine, INRIA */
+/*
+    This file is part of mldonkey.
+
+    mldonkey is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    mldonkey is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with mldonkey; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*/
 
 
 #include "../../utils/lib/os_stubs.h"
@@ -179,27 +197,6 @@ void os_set_nonblock(OS_SOCKET fd)
   }
 }
 
-/*******************************************************************
-
-
-                         gettimeofday
-
-
-*******************************************************************/
-
-//http://lists.gnu.org/archive/html/bug-gnu-chess/2004-01/msg00020.html
-void gettimeofday(struct timeval* p, void* tz /* IGNORED */){
-   union {
-     long long ns100; /*time since 1 Jan 1601 in 100ns units */
-     FILETIME ft;
-   } _now;
-
-   GetSystemTimeAsFileTime( &(_now.ft) );
-   p->tv_usec=(long)((_now.ns100 / 10LL) % 1000000LL );
-   p->tv_sec= (long)((_now.ns100-(116444736000000000LL))/10000000LL);
-   return;
-}
-
 /*******************************************************************
 
 
@@ -479,3 +476,10 @@ value os_set_console_title(value buf)
    SetConsoleTitle((LPCTSTR)pbuf);
    return Val_unit;
 }
+
+int os_fsync(OS_FD fd)
+{
+   int r = FlushFileBuffers(fd);
+   return (0 == r ? -1 : 0);
+}
+
