@@ -94,6 +94,7 @@ let fork_and_exec cmd ?vars args =
   | id -> ignore (snd(Unix.waitpid [] id))
       
 let setuid = Unix.setuid
+let setgid = Unix.setgid
 let set_close_on_exec = Unix.set_close_on_exec
 let set_signal signal f = Sys.set_signal signal f
   
@@ -108,6 +109,8 @@ external glibc_version : unit -> string = "glibc_version"
 let glibc_version_num () =
   begin
     try
-      let s = Printf.sprintf "%s" (glibc_version ()) in s
+      glibc_version ()
     with e -> ""
   end
+  
+let set_console_title buf = ()  (* fake function for DriverMain.mingw_second_timer *)
