@@ -167,7 +167,10 @@ module FileOption = struct
           set_file_priority file priority;
 
           if !verbose && !CommonGlobals.is_startup_phase then
-            lprintf_nl "Started download of %s" (file_best_name file);
+            lprintf_nl "Started download of %s, user:group %s:%s"
+              (file_best_name file)
+              file_user.user_name
+              (user2_print_group file_group);
 
           file
       | _ -> assert false
@@ -1135,7 +1138,9 @@ let backup_tar archive files =
 	    let stat = Unix.stat arg in
 	    let size = stat.Unix.st_size in
 	    if size > Sys.max_string_length then
-	      failwith (Printf.sprintf "Tar: file %s too big, limit %d byte" arg Sys.max_string_length);
+	      failwith (Printf.sprintf
+                  "Tar: file %s too big, system limit %d byte, use .zip to avoid this limit"
+                    arg Sys.max_string_length);
 	    let header = 
 	      { Tar.t_name = arg;
 	      t_mode = stat.Unix.st_perm;
@@ -1158,18 +1163,15 @@ let backup_tar archive files =
 	    header, s) in
 	Tar.output otar header s
       with
-      | (Gzip.Error "error during compression") as e when Autoconf.windows && arg = "fasttrack.ini" ->
-	  (* for whatever reason this error is raised on Windows,
-             but fasttrack.ini is stored correctly *)
-	  if !verbose then
-            lprintf_nl "Tar: Windows specific pseudo error %s in %s" (Printexc2.to_string e) arg
       | e ->
 	  failed_files := arg :: !failed_files;
-	  lprintf_nl "Tar: error %s in %s" (Printexc2.to_string e) arg
-    ) files));
-    if !failed_files <> [] then
-      failwith (Printf.sprintf "Tar: error backing up %s"
-	(String.concat " " (List.rev !failed_files)))
+	  lprintf_nl "Tar: skipping %s, error %s" arg (Printexc2.to_string e)
+      ) files
+    )
+  );
+  if !failed_files <> [] then
+    failwith (Printf.sprintf "Tar: skipped %s due to backup errors"
+      (String.concat " " (List.rev !failed_files)))
 
 let backup_options () =
   let counter = ref 1 in
@@ -1195,17 +1197,21 @@ let backup_options () =
   in
   begin
     try
-    let archive = Filename.concat "old_config" (backup_prefix ^ Date.reverse (Unix.time ()) ^ format) in
-      let files = List.sort (fun o -> compare o) (List.filter (fun o ->
+      let archive =
+        Filename.concat "old_config" (backup_prefix ^ Date.reverse (Unix.time ()) ^ format)
+      in
+      let files =
+        List.sort (fun o -> compare o) (List.filter (fun o ->
 	  String.lowercase (Filename2.last_extension o) = ".ini"
-	  && o <> "file_sources.ini") 
-	    (Unix2.list_directory file_basedir)) in
+	  && o <> "file_sources.ini")
+	    (Unix2.list_directory file_basedir))
+      in
       begin
         match (Filename2.last_extension archive) with
           ".zip" -> backup_zip archive files
         | _ -> backup_tar archive files
       end
-    with e -> lprintf_nl "Exception %s while options backup" (Printexc2.to_string e); raise e
+    with e -> lprintf_nl "Exception %s while options backup" (Printexc2.to_string e)
   end;
   lprintf_nl (_b "Options backup as %s correctly saved") format
              
@@ -1233,6 +1239,12 @@ let _ =
     shorten_all_file_filenames !!max_filenames
   );
 
+  option_hook max_upload_slots (fun _ ->
+      if !!max_upload_slots < 3 then
+        max_upload_slots =:= 3;
+      networks_iter (fun n -> network_check_upload_slots n)
+    );
+
   let max_opened_connections_pass = ref 0 in
   option_hook max_opened_connections (fun _ ->
 
