@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with mldonkey; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include <stdio.h>
@@ -348,6 +348,7 @@ static void * hasher_thread(void * arg)
 
       case METHOD_TIGER:
         bsize = tiger_block_size(job_len);
+        os_lseek(job_fd, job_begin_pos, SEEK_SET);
         tiger_tree_fd(job_fd, job_len, 0, bsize, p_job_result);
       break;
 
