@@ -100,6 +100,7 @@ let impl_server_info impl =
     T.server_addr = Ip.addr_of_ip Ip.null;
     T.server_port = 0;
     T.server_realport = 0;
+    T.server_country_code = None;
     T.server_score = 0;
     T.server_tags = [];
     T.server_nusers = 0L;
@@ -272,7 +273,7 @@ let server_find (num : int) =
 
 let server_blocked s =
   let info = server_info s in
-  !Ip.banned (Ip.ip_of_addr info.G.server_addr) <> None
+  !Ip.banned (Ip.ip_of_addr info.G.server_addr, info.G.server_country_code) <> None
 
 let server_connect s =
   if not (server_blocked s) then
@@ -376,27 +377,27 @@ let server_print_html_header buf ext =
 -100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>";
 
     html_mods_table_header buf "serversTable" (Printf.sprintf "servers%s" ext) ([
-    ( "1", "srh", "Server number", "#" ) ;
-    ( "0", "srh", "Connect|Disconnect", "C/D" ) ;
-    ( "0", "srh", "Remove", "Rem" ) ;
-    ( "0", "srh", "Preferred", "P" ) ;
-    ( "0", "srh", "Master servers", "M" ) ;
-    ( "0", "srh", "[Hi]gh or [Lo]w ID", "ID" ) ;
-    ( "0", "srh", "Network name", "Network" ) ;
-    ( "0", "srh", "Connection status", "Status" ) ;
-    ( "0", "srh br", "IP address", "IP address" ) ;
-    ] @ (if !Geoip.active then [( "0", "srh br", "Country Code/Name", "CC" )] else []) @ [
-    ( "1", "srh ar", "Number of connected users", "Users" ) ;
-    ( "1", "srh ar br", "Max number of users", "MaxUsers" ) ;
-    ( "1", "srh ar br", "LowID users", "LowID" ) ;
-    ( "1", "srh ar", "Number of files indexed on server", "Files" );
-    ( "1", "srh ar br", "Number of published files on server", "Publ" );
-    ( "1", "srh ar", "Soft file limit", "Soft" ) ;
-    ( "1", "srh ar br", "Hard file limit", "Hard" ) ;
-    ( "0", "srh ar br", "Ping (ms)", "Ping" ) ;
-    ( "0", "srh br", "Server version", "Version" ) ;
-    ( "0", "srh", "Server name", "Name" ) ;
-    ( "0", "srh", "Server details", "Details" ) ])
+    ( Num, "srh", "Server number", "#" ) ;
+    ( Str, "srh", "Connect|Disconnect", "C/D" ) ;
+    ( Str, "srh", "Remove", "Rem" ) ;
+    ( Str, "srh", "Preferred", "P" ) ;
+    ( Str, "srh", "Master servers", "M" ) ;
+    ( Str, "srh", "[Hi]gh or [Lo]w ID", "ID" ) ;
+    ( Str, "srh", "Network name", "Network" ) ;
+    ( Str, "srh", "Connection status", "Status" ) ;
+    ( Str, "srh br", "IP address", "IP address" ) ;
+    ] @ (if Geoip.active () then [( Str, "srh br", "Country Code/Name", "CC" )] else []) @ [
+    ( Num, "srh ar", "Number of connected users", "Users" ) ;
+    ( Num, "srh ar br", "Max number of users", "MaxUsers" ) ;
+    ( Num, "srh ar br", "LowID users", "LowID" ) ;
+    ( Num, "srh ar", "Number of files indexed on server", "Files" );
+    ( Num, "srh ar br", "Number of published files on server", "Publ" );
+    ( Num, "srh ar", "Soft file limit", "Soft" ) ;
+    ( Num, "srh ar br", "Hard file limit", "Hard" ) ;
+    ( Num, "srh ar br", "Ping (ms)", "Ping" ) ;
+    ( Str, "srh br", "Server version", "Version" ) ;
+    ( Str, "srh", "Server name", "Name" ) ;
+    ( Str, "srh", "Server details", "Details" ) ])
 
 let server_print s o =
   let impl = as_server_impl s in
@@ -408,7 +409,7 @@ let server_print s o =
           lprintf_nl "Exception %s in server_info (%s)\n"
             (Printexc2.to_string e) n.network_name;
           raise e in
-    let cc,cn = Geoip.get_country (Ip.ip_of_addr info.G.server_addr) in
+    let cc,cn = Geoip.get_country_code_name info.G.server_country_code in
     let buf = o.conn_buf in
   
   if use_html_mods o then begin
@@ -426,7 +427,7 @@ let server_print s o =
     Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\""
     (html_mods_cntr ());
 
-    (if !!html_mods_use_js_tooltips then
+    if !!html_mods_use_js_tooltips then
        Printf.bprintf buf " onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s %s<br>%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
         info.G.server_name ip_port_string 
 	(match info.G.server_features with
@@ -434,10 +435,10 @@ let server_print s o =
 	| Some f -> "server features: " ^ f)
        !!html_mods_js_tooltips_wait
        !!html_mods_js_tooltips_timeout
-       !!html_mods_js_tooltips_wait);
+       !!html_mods_js_tooltips_wait
+    else Printf.bprintf buf "\\>";
 
-    Printf.bprintf buf 
-" \\<td class=\\\"srb\\\" %s \\>%d\\</td\\> %s %s %s"
+    Printf.bprintf buf " \\<td class=\\\"srb\\\" %s \\>%d\\</td\\> %s %s %s"
     (match impl.impl_server_state with
         Connected _ -> 
             Printf.sprintf "title=\\\"Server Banner\\\"
@@ -514,14 +515,13 @@ let server_print s o =
       | _ -> string_of_connection_state impl.impl_server_state
     in
 
-    let cc,cn = Geoip.get_country (Ip.ip_of_addr info.G.server_addr) in
     html_mods_td buf ([
       ("", "srb", if info.G.server_master then "M" else "-");
       (id_title, "sr", id_text);
       ("", "sr", n.network_name);
       ("", "sr", server_state_string);
       ("", "sr br", ip_port_string);
-      ] @ (if !Geoip.active then [(cn, "sr br", CommonPictures.flag_html cc)] else []) @ [
+      ] @ (if Geoip.active () then [(cn, "sr br", CommonPictures.flag_html cc)] else []) @ [
       ("", "sr ar", if info.G.server_nusers = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_nusers);
       ("", "sr ar br", if info.G.server_max_users = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_max_users);
       ("", "sr ar br", if info.G.server_lowid_users = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_lowid_users);
@@ -559,7 +559,7 @@ onClick=\\\"location.href='submit?q=server_shares+%d'\\\"\\>%d\\</TD\\>"
             then "(" ^ (string_of_int info.G.server_realport) ^ ")" 
             else ""))
           (info.G.server_name)
-	  (if !Geoip.active then Printf.sprintf "%33s/%-2s%9s" cn cc "" else "")
+	  (if Geoip.active () then Printf.sprintf "%33s/%-2s%9s" cn cc "" else "")
           (info.G.server_nusers)
           (info.G.server_nfiles)
           (if server_blocked s 
