@@ -226,8 +226,6 @@ let _ =
   network.op_network_connected <- (fun _ -> false);
   network.op_network_is_enabled <- (fun _ -> raise IgnoreNetwork);
   network.op_network_update_options <- (fun _ -> raise IgnoreNetwork);
-  (* op_network_info defined twice ?? *)
-  network.op_network_info <- (fun _ -> raise Not_found);
   network.op_network_info <- (fun n ->
       {
         network_netnum = network.network_num;
@@ -751,23 +749,6 @@ let streaming_amount () =
 let streaming_left = ref (streaming_amount ())
 let streaming_time = (ref None : float option ref)
 
-let has_upload = ref 0
-let upload_credit = ref 0
-
-
-let can_write_len sock len =
-  let bool1 = can_write_len sock len in
-(* changed 2.5.24: Don't put in a socket more than 10 seconds of upload. *)
-(* ...since the socket shares available bandwidth with others, 
-   that test has no physical meaning. Should it simply be removed ? *)
-  let bool2 = not_buffer_more sock (int_of_float !CommonGlobals.payload_bandwidth * 10) in
-(*  lprintf "can_write_len %b %b\n" bool1 bool2; *)
-  let b = bool1 && bool2 in
-(*  if not b then
-      lprintf "bool1 %b len %d bool2 %b upload %d\n" 
-        bool1 len bool2 upload_rate; *)
-  b
-
 let next_uploads () =
 
   let rec next_uploads_aux () =
@@ -801,21 +782,26 @@ let next_uploads () =
       lprintf "streaming_left %d\n" !streaming_left;
     end; *)
   (* buffer empties with time... *)
+  (* FIXME wall-clock time is not needed here and causes problems when clock jumps *)
   let new_streaming_time = BasicSocket.current_time () in
-  let deltat = (match !streaming_time with
+  let deltat' = (match !streaming_time with
     | None -> 0.
     | Some t -> new_streaming_time -. t) in
-  streaming_left := !streaming_left + 
-    (int_of_float (!CommonGlobals.payload_bandwidth *. deltat));
+  (* stay sane no matter what *)
+  let deltat = min 10. (max 0. deltat') in
+  if abs_float (deltat -. deltat') > epsilon_float then
+    lprintf_nl "Detected clock jump. deltat %f adjusted to %f" deltat' deltat;
+  (* do not overflow *)
+  let deltab = !CommonGlobals.payload_bandwidth *. deltat in
+  if deltab > float max_int then
+    lprintf_nl "OVERFLOW deltab %f, ignored" deltab
+  else
+    streaming_left := !streaming_left + (int_of_float deltab);
+(*   lprintf_nl "next_uploads %f %f %d %d %d" new_streaming_time deltat !streaming_left (streaming_amount()) (Fifo.length upload_clients); *)
   streaming_left := min !streaming_left (streaming_amount ());
   streaming_time := Some new_streaming_time;
   next_uploads_aux ()
 
-let reset_upload_timer () = ()
-
-let reset_upload_timer _ =
-  reset_upload_timer ()
-
 let upload_credit_timer _ =
   if !has_upload = 0 then
     (if !upload_credit < 300 then incr upload_credit)
@@ -965,6 +951,7 @@ let dynamic_refill_upload_slots () =
 let turn = ref (-1)
 
 let refill_upload_slots () =
+  if !CommonGlobals.has_upload = 0 then begin
   incr turn;
   if !turn = 5 then
     turn := 0;
@@ -975,6 +962,7 @@ let refill_upload_slots () =
   end else
     (* call every 1s *)
     static_refill_upload_slots ()
+  end
 
 let consume_bandwidth len =
   streaming_left := !streaming_left - len
