@@ -105,6 +105,7 @@ let eval auth cmd o =
   let buf = o.conn_buf in
   let cmd =
     if String2.check_prefix cmd "ed2k://" ||
+       String2.check_prefix cmd "magnet:" ||
        String2.check_prefix cmd "ftp://" ||
        String2.check_prefix cmd "http://" then "dllink " ^ cmd
     else if String2.check_prefix cmd "fha://" then "ovlink " ^ cmd
@@ -953,7 +954,20 @@ let http_handler o t r =
   CommonInteractive.display_vd := false;
   CommonInteractive.display_bw_stats := false;
   clear_page buf;
-  if !Http_server.verbose && r.get_url.Url.short_file <> "" then
+  let short_file =
+    let file = r.get_url.Url.short_file in
+    match !!http_root_url with
+    | "" | "/" -> `File file
+    | root ->
+      let root = if not (String2.starts_with root "/") then "/" ^ root else root in
+      (* we want to treat "/root" requests as invalid and redirect them to "/root/" *)
+      let root_dir = if root <> "" && root.[String.length root - 1] = '/' then root else root ^ "/" in
+      if String2.starts_with ("/"^file) root_dir then
+        `File (String2.after file (String.length root_dir - 1))
+      else
+        `Redirect root_dir
+  in
+  if !Http_server.verbose && short_file <> `File "" then
     lprintf_nl "received URL %s %s"
       r.get_url.Url.short_file
       (let b = Buffer.create 100 in
@@ -961,12 +975,10 @@ let http_handler o t r =
 	 if Buffer.contents b <> "" then Printf.sprintf "(%s)" (Buffer.contents b) else "");
 
   let user = if r.options.login = "" then (admin_user ()).CommonTypes.user_name else r.options.login in
-  if not (valid_password user r.options.passwd) || (r.get_url.Url.short_file = "logout") then begin
+  if not (valid_password user r.options.passwd) || (short_file = `File "logout") then begin
       clear_page buf;
       http_file_type := HTM;
-      let _, error_text_long, head = Http_server.error_page "401" "" ""
-	(Ip.to_string (TcpBufferedSocket.my_ip r.sock))
-	(string_of_int !!http_port) None in
+      let _, error_text_long, head = Http_server.error_page Unauthorized (TcpBufferedSocket.my_ip r.sock) !!http_port in
       Buffer.add_string buf error_text_long;
       r.reply_head <- head;
       http_add_html_header r;
@@ -983,7 +995,18 @@ let http_handler o t r =
             user.ui_http_conn <- Some oo; oo
       in
       try
-        match r.get_url.Url.short_file with
+        match short_file with
+        | `Redirect url ->
+          let _, error_text_long, head = Http_server.error_page (Moved url)
+            (TcpBufferedSocket.my_ip r.sock)
+            !!http_port
+          in
+          r.reply_head <- head;
+          add_reply_header r "Location" url;
+          http_add_html_header r;
+          Buffer.add_string buf error_text_long
+        | `File short_file ->
+        match short_file with
         | "wap.wml" ->
             begin
               clear_page buf;
@@ -1519,10 +1542,9 @@ let http_handler o t r =
         | s ->  http_send_bin_pictures r buf (String.lowercase s)
       with
       | Not_found ->
-	  let _, error_text_long, head = Http_server.error_page "404" "" ""
-			(Ip.to_string (TcpBufferedSocket.my_ip r.sock))
-			(string_of_int !!http_port)
-			(Some (Url_not_found r.get_url.Url.full_file)) in
+	  let _, error_text_long, head = Http_server.error_page (Not_Found r.get_url.Url.full_file)
+			(TcpBufferedSocket.my_ip r.sock) !!http_port
+    in
 	  r.reply_head <- head;
           http_add_html_header r;
 	  Buffer.add_string buf error_text_long
