@@ -237,6 +237,9 @@ let send_event gui ev =
   | Console_message_event msg ->
       gui_send gui (P.Console msg)
 
+  | Root_console_message_event msg ->
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then gui_send gui (P.Console msg)
+
   | Network_info_event n ->
       gui_send gui (P.Network_info (network_info n))
 
@@ -743,7 +746,11 @@ let gui_reader (gui: gui_record) t _ =
                 if not (networks_iter_until_true
                     (fun n ->
                        try
-                         let s,r = network_parse_url n url gui.gui_conn.conn_user.ui_user in r
+                         let s,r =
+                            network_parse_url n url
+                              gui.gui_conn.conn_user.ui_user
+                              gui.gui_conn.conn_user.ui_user.user_default_group
+                         in r
                        with e ->
                          lprintf "Exception %s for network %s\n"
                            (Printexc2.to_string e) (n.network_name);
@@ -1121,8 +1128,6 @@ let gui_reader (gui: gui_record) t _ =
       gui_send gui (Console (Printf.sprintf "Failure: %s\n" s))
   | Torrent_started s ->
       gui_send gui (Console (Printf.sprintf "\nInfo: Torrent %s started\n" s))
-  | Torrent_can_not_be_used s ->
-      gui_send gui (Console (Printf.sprintf "\nError: Torrent %s does not have valid tracker URLs\n" s))
   | Torrent_already_exists s ->
       gui_send gui (Console (Printf.sprintf "\nError: Torrent %s is already in download queue\n" s))
   | e ->
@@ -1192,7 +1197,7 @@ let gui_handler t event =
       let from_ip = Ip.of_inet_addr from_ip in
       if not !verbose_no_login then lprintf_nl "GUI connection from %s" (Ip.to_string from_ip);
       (match Ip_set.match_ip !allowed_ips_set from_ip with
-      |	Some br ->
+      |	true ->
         
         let module P = GuiProto in
         let token = create_token unlimited_connection_manager in
@@ -1219,7 +1224,7 @@ let gui_handler t event =
             close sock Closed_for_overflow);
         (* sort GUIs in increasing order of their num *)
         
-      | None ->
+      | false ->
           if not !verbose_no_login then lprintf_nl "GUI connection from %s rejected (see allowed_ips setting)"
             (Ip.to_string from_ip);
           Unix.close s)
@@ -1231,7 +1236,7 @@ let gift_handler t event =
       let from_ip = Ip.of_inet_addr from_ip in
       lprintf "Gift: Connection from %s\n" (Ip.to_string from_ip);
       (match Ip_set.match_ip !allowed_ips_set from_ip with
-      | Some br ->
+      | true ->
         
         let module P = GuiProto in
         let token = create_token unlimited_connection_manager in
@@ -1257,7 +1262,7 @@ let gift_handler t event =
             close sock Closed_for_overflow);
         (* sort GUIs in increasing order of their num *)
         
-      | None ->
+      | false ->
           lprintf "Connection from IP %s not allowed\n"
             (Ip.to_string from_ip);
           Unix.close s)
@@ -1348,7 +1353,7 @@ let rec update_events list =
               update_result_info r;
               gui.gui_new_events <- event :: gui.gui_new_events
               
-          | Console_message_event msg ->  
+          | Console_message_event msg | Root_console_message_event msg ->
               Fifo.put console_messages event;
               if Fifo.length console_messages > !!gui_log_size then
                   ignore (Fifo.take console_messages);
