@@ -129,7 +129,7 @@ let get_source_pix client_type =
   
 let save_menu_items file =
   List.map
-    (fun (name,_) ->
+    (fun name ->
       `I (name, 
         (fun _ -> 
             match file.data.gfile_state with
@@ -183,6 +183,7 @@ let client_to_general_state state file_num =
     | Connecting  -> CConnecting
     | NewHost -> CNewHost
     | Connected_initiating -> CConnected_initiating
+    | ServerFull -> CNotConnected (BasicSocket.Closed_connect_failed,0)
     | NotConnected (p,n) -> CNotConnected (p,n)
     | RemovedHost -> CRemovedHost
     | BlackListedHost -> CBlackListedHost
@@ -467,7 +468,7 @@ class box columns sel_mode () =
               Printf.sprintf "%5.1f" 
                  (Int64.to_float f.data.gfile_downloaded /. Int64.to_float f.data.gfile_size *. 100.)
               else ""
-            else fst (List.hd f.data.gfile_names)
+            else List.hd f.data.gfile_names
       |	Col_file_rate ->
           if (List.length f.data.gfile_num) = 1 then
             if f.data.gfile_download_rate > 0. then
@@ -1155,7 +1156,7 @@ class box_downloads wl_status () =
               if icons_are_used then
                 Printf.sprintf "Client %d" client_num
                 else "   |-- " ^ (Printf.sprintf "Client %d" client_num);
-            gfile_names = ["", noips()];
+            gfile_names = [""];
             gfile_md4 = file.data.gfile_md4;
             gfile_size = Int64.of_string "0";
             gfile_downloaded = Int64.of_string "0";
@@ -1387,10 +1388,9 @@ class box_downloads wl_status () =
                                                      then c.client_software
                                                      else (c.client_software ^ 
                                                            " - " ^
-                                                           c.client_emulemod)),
-                                                   noips()];
-                        child.data.gfile_size <- c.client_uploaded;
-                        child.data.gfile_downloaded <- c.client_downloaded;
+                                                           c.client_emulemod))];
+                        child.data.gfile_size <- c.client_total_uploaded;
+                        child.data.gfile_downloaded <- c.client_total_downloaded;
                         child.data.gfile_state  <- client_to_general_state c.client_state (List.hd f.data.gfile_num);
                         child.data.gfile_chunks <- f.data.gfile_chunks;
                         child.data.gfile_name <- 
@@ -1410,11 +1410,10 @@ class box_downloads wl_status () =
                 begin
                   f.data.gfile_names <- [c.client_software ^ 
                                          " - " ^
-                                         c.client_emulemod,
-                                         noips()];
+                                         c.client_emulemod];
                   f.data.gfile_state <- client_to_general_state c.client_state file_num;
-                  f.data.gfile_size <- c.client_uploaded;
-                  f.data.gfile_downloaded <- c.client_downloaded;
+                  f.data.gfile_size <- c.client_total_uploaded;
+                  f.data.gfile_downloaded <- c.client_total_downloaded;
                   if f.data.gfile_type <> c.client_type then
                     begin
                       f.data.gfile_type <- c.client_type;                 
