@@ -67,8 +67,7 @@ let disconnect gui reason =
 
 let send t =
   match !connection with
-    None -> 
-      lprintf "Message not sent since not connected\n";
+    None -> ()
   | Some sock ->
       GuiEncoding.gui_send (GuiEncoding.from_gui from_gui_protocol_used) sock t
           
@@ -227,7 +226,7 @@ let scan_ports () =
             scan_port next (i+1) max);
         let proto = ref 0 in
         let nets = ref [] in
-        let console = ref "" in
+        let _console = ref "" in
         TcpBufferedSocket.set_reader sock (fun sock nread ->
             GuiDecoding.gui_cut_messages
               (fun opcode s ->
