@@ -50,9 +50,13 @@ let user_agent =
 
 let make_request url =
   let module H = Http_client in
+  let auth = match !!O.gtk_connection_http_proxy_login with
+  | "" -> None
+  | _ -> Some (!!O.gtk_connection_http_proxy_login, !!O.gtk_connection_http_proxy_password)
+  in
   let proxy =
     if !!O.gtk_connection_http_use_proxy
-      then Some (!!O.gtk_connection_http_proxy_server, !!O.gtk_connection_http_proxy_port)
+      then Some (!!O.gtk_connection_http_proxy_server, !!O.gtk_connection_http_proxy_port, auth)
       else None
   in
   let r = {
@@ -166,16 +170,15 @@ let get_html url_name f progress =
       try
         (try Sys.remove filename with _ -> ());
         File.from_string filename _s;
-      with _ -> (lprintf_nl2 "failed in H.wget_string")
+      with _ -> (lprintf_nl "failed in H.wget_string")
     end;
-    let in_chan = open_in_bin filename in
-    let lexb = Lexing.from_channel in_chan in
-    let dl = GuiNetHtml.parse_document
-               ~return_declarations:true ~return_pis:true
-               ~return_comments:true lexb
-    in
-    f dl;
-    close_in in_chan) (fun n m -> progress n m)
+    Unix2.tryopen_read_bin filename (fun in_chan ->
+      let lexb = Lexing.from_channel in_chan in
+      let dl = GuiNetHtml.parse_document
+        ~return_declarations:true ~return_pis:true
+        ~return_comments:true lexb
+      in
+      f dl)) (fun n m -> progress n m)
 
 
 (*************************************************************************)
@@ -247,7 +250,7 @@ let parse_razorback_stats stats md4 referer on_png_completed on_not_found progre
                         File.from_string filename ___s;
                         stats.stats_file_history <- filename;
                         on_png_completed ();
-                      with _ -> (lprintf_nl2 "failed in H.wget_string")) (fun n m -> progress progress_desc n m)
+                      with _ -> (lprintf_nl "failed in H.wget_string")) (fun n m -> progress progress_desc n m)
                   end
               ) sl;
               iter __dl
