@@ -170,6 +170,15 @@ let result_menu sel =
            `I (!M.qT_me_force_download, force_download sel);
           ]
 
+(*************************************************************************)
+(*                                                                       *)
+(*                         result_on_double_click                        *)
+(*                                                                       *)
+(*************************************************************************)
+
+let result_on_double_click key =
+  ignore (download [key] ())
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         filter_result                                 *)
