@@ -70,7 +70,7 @@ let decode_torrent s =
   let file_info = ref (List []) in
   let file_name = ref "" in
   let file_torrent_filename = ref "" in
-  let file_name_utf8 = ref "" in
+  let file_name_utf8 = ref None in
   let file_piece_size = ref zero in
   let file_pieces = ref "" in
   let file_comment = ref "" in
@@ -78,12 +78,14 @@ let decode_torrent s =
   let file_creation_date = ref zero in
   let file_modified_by = ref "" in
   let file_encoding = ref "" in
+  let file_codepage = ref zero in
   let file_ed2k_hash = ref "" in
-  let file_is_private = ref zero in
+  let file_is_private = ref false in
   let file_aps = ref (List []) in
   let file_dht_backup_enable = ref zero in
   let length = ref zero in
   let file_files = ref [] in
+  let file_files_utf8 = ref [] in
 
   let parse_files files =
     let current_pos = ref zero in
@@ -91,6 +93,7 @@ let decode_torrent s =
         match v with
           Dictionary list ->
             let current_file = ref "" in
+            let current_file_utf8 = ref "" in
             let current_length = ref zero in
             let length_set = ref false in
 
@@ -107,30 +110,28 @@ let decode_torrent s =
 
             List.iter (fun (key, value) ->
                 match key, value with
-                  String "path", List path ->
-                    if !current_file = "" then begin
-                      current_file := path_list_to_string path;
-                      if !verbose_msg_servers then
-                        lprintf_nl "[BT] Parsed a new path: [%s]" !current_file
-                    end   
-                | String "path.utf-8", List path_utf8 -> 
-                    current_file := path_list_to_string path_utf8;
+                | "path", List path ->
+                    current_file := path_list_to_string path;
+                    if !verbose_msg_servers then
+                      lprintf_nl "[BT] Parsed a new path: [%s]" !current_file
+                | "path.utf-8", List path_utf8 -> 
+                    current_file_utf8 := path_list_to_string path_utf8;
                     if !verbose_msg_servers then
                       lprintf_nl "[BT] Parsed path.utf-8: [%s]" !current_file
-                | String "length", Int n ->
+                | "length", Int n ->
                     length := !length ++ n;
                     current_length := n;
                     length_set := true
 
-                | String key, _ ->
+                | key, _ ->
                     if !verbose_msg_servers then lprintf_nl "[BT] other field [%s] with value [%s] in files" key (Bencode.print value)
-                | _ ->
-                    lprintf_nl "[BT] other field in files"
             ) list;
 
             assert (!length_set);
-            assert (!current_file <> "");
+            assert (!current_file <> "" || !current_file_utf8 <> "");
             file_files := (!current_file, !current_length) :: !file_files;
+            if !current_file_utf8 <> "" then
+              file_files_utf8 := (!current_file_utf8, !current_length) :: !file_files_utf8;
             current_pos := !current_pos ++ !current_length
 
         | _ -> assert false
@@ -142,11 +143,11 @@ let decode_torrent s =
       Dictionary list ->
         List.iter (fun (key, value) ->
             match key, value with
-              String "announce", String tracker_url ->
+            | "announce", String tracker_url ->
                if !verbose_msg_servers then
-                 lprintf_nl "[BT] New tracker added :%s" tracker_url;
+                 lprintf_nl "[BT] New tracker added: %s" tracker_url;
                 announce := tracker_url
-            | String "announce-list", List list ->
+            | "announce-list", List list ->
                 List.iter (fun url_list ->
                     let next_urls = ref [] in
                     match url_list with
@@ -166,7 +167,7 @@ let decode_torrent s =
                             announce_list := !next_urls @ !announce_list
                           end
                         else
-                          announce_list := List.hd !next_urls :: !announce_list
+                          announce_list := (try List.hd !next_urls with _ -> "") :: !announce_list
                     | _ ->
                         lprintf_nl "[BT] unknown field in announce list"
                     ) list;
@@ -175,138 +176,152 @@ let decode_torrent s =
                       List.iter (fun url ->
                         lprintf_nl "[BT] New tracker added :%s" url
                         ) !announce_list
-            | String "info", ((Dictionary list) as info) ->
+            | "info", ((Dictionary list) as info) ->
 
                 file_info := info;
                 List.iter (fun (key, value) ->
                     match key, value with
-                    | String "files", List files ->
+                    | "files", List files ->
                         parse_files files
-                    | String "length", Int n ->
+                    | "length", Int n ->
                         length := n
-                    | String "name", String name ->
+                    | "name", String name ->
                         file_name := name
-                    | String "piece length", Int n ->
+                    | "piece length", Int n ->
                         file_piece_size := n
-                    | String "pieces", String pieces ->
+                    | "pieces", String pieces ->
                         file_pieces := pieces
-                    | String "ed2k", String string_ed2k ->
+                    | "ed2k", String string_ed2k ->
                         if !!enable_donkey then
                           file_ed2k_hash := string_ed2k;
                       (* TODO: Add new ed2k download if ed2k hash is available,
                                then merge it with current download *)
-                    | String "sha1", String string_sha1 -> ()
+                    | "sha1", String string_sha1 -> ()
                       (* TODO: Parse sha1 hash *)
 
-                    | String "publisher", String created_by ->
+                    | "publisher", String created_by ->
                       file_created_by := created_by
-                    | String "publisher-url", String publisher_url ->
+                    | "publisher-url", String publisher_url ->
                       file_created_by := !file_created_by ^ " @ " ^ publisher_url
 
-                    | String "name.utf-8", String name_utf8 ->
-                        file_name_utf8 := name_utf8
+                    | "name.utf-8", String name_utf8 ->
+                        file_name_utf8 := Some name_utf8
 
-                    | String "publisher.utf-8", String publisher_utf8 -> ()
-                    | String "publisher-url.utf-8", String publisher_url_utf8 -> ()
+                    | "publisher.utf-8", String publisher_utf8 -> ()
+                    | "publisher-url.utf-8", String publisher_url_utf8 -> ()
 
-                    | String "private", Int n ->
-                        (* TODO: if set to 1, only accept peers from tracker *)
-                        file_is_private := n;
-                        if !verbose_msg_servers &&
-                          Int64.to_int !file_is_private = 1 then
+                    | "private", Int n ->
+                        file_is_private := n <> 0L;
+                        if !verbose_msg_servers && !file_is_private then
                             lprintf_nl "[BT] torrent is private"
-                    | String key, _ ->
+                    | key, _ ->
                         if !verbose_msg_servers then
                           lprintf_nl "[BT] found other field [%s] with value [%s] in info" key (Bencode.print value)
-                    | _ ->
-                        lprintf_nl "[BT] other field in info"
                 ) list
 
-            | String "comment", String comment
-            | String "comment.utf-8", String comment ->
+            | "comment", String comment
+            | "comment.utf-8", String comment ->
               file_comment := comment
             (* Next 2 strings are after info sometimes  *)
-            | String "publisher", String created_by ->
+            | "publisher", String created_by ->
               file_created_by := created_by
-            | String "publisher-url", String publisher_url ->
+            | "publisher-url", String publisher_url ->
               file_created_by := !file_created_by ^ " @ " ^ publisher_url
 
-            | String "created by", String created_by ->
+            | "created by", String created_by ->
               file_created_by := created_by
-            | String "creation date", Int creation_date ->
+            | "creation date", Int creation_date ->
               file_creation_date := creation_date
-            | String "modified-by", String modified_by ->
+            | "modified-by", String modified_by ->
               file_modified_by := modified_by
-            | String "encoding", String encoding ->
+            | "encoding", String encoding ->
               file_encoding := encoding
-            | String "torrent filename", String torrent_filename ->
+            | "codepage", Int codepage ->
+              file_codepage := codepage
+            | "torrent filename", String torrent_filename ->
               file_torrent_filename := torrent_filename
-            | String "nodes", nodes -> ()
+            | "nodes", nodes -> ()
               (* TODO : nodes is a list of DHT Network nodes ,parse and use them *)
 
 (*
               file_nodes := nodes
 *)
 
-            | String "azureus_properties", ((Dictionary list) as azureus_properties) ->
+            | "azureus_properties", ((Dictionary list) as azureus_properties) ->
                 file_aps := azureus_properties;
                 List.iter (fun (key, value) ->
                     match key, value with
-                    | String "dht_backup_enable", Int n ->
+                    | "dht_backup_enable", Int n ->
                         file_dht_backup_enable := n;
                         if !verbose_msg_servers &&
                           Int64.to_int !file_dht_backup_enable = 1 then
                             lprintf_nl "[BT] azureus properties : Torrent has dht backup"
-                    | String key, _ ->
+                    | key, _ ->
                         if !verbose_msg_servers then
                           lprintf_nl "[BT] found other field [%s] with value [%s] in azureus properties" key (Bencode.print value)
-                    | _ ->
-                        lprintf_nl "[BT] other field in azureus properties"
                 ) list
-            | String key, _ ->
+            | key, _ ->
                 if !verbose_msg_servers then lprintf_nl "[BT] found other field [%s] with value [%s] after info" key (Bencode.print value)
-            | _ ->
-                lprintf_nl "[BT] other field after info"
         ) list
     | _ -> assert false
   end;
 
-  assert (!announce <> "");
-  assert (!file_name <> "");
+(* Convert codepage number to Charset name, for example: 936 -> CP936 *)
+  if !file_codepage <> 0L && !file_encoding = "" then
+    file_encoding := "CP" ^ (Int64.to_string !file_codepage);
+
+  let real_file_name =
+    match !file_name_utf8 with
+    | None -> Charset.safe_convert !file_encoding !file_name
+    | Some name -> name
+  in
+  assert (real_file_name <> "");
   assert (!file_piece_size <> zero);
   assert (!file_pieces <> "");
+  assert (!file_info = Bencode.decode (Bencode.encode ~strict:false !file_info));
 
-  assert (!file_info = Bencode.decode (Bencode.encode !file_info));
-
-  let file_id = Sha1.string (Bencode.encode !file_info) in
-  let npieces =
-    1+ Int64.to_int ((!length -- one) // !file_piece_size)
-  in
-(*            lprintf "npieces %d length %Ld piece %Ld %d\n"
-              npieces !length !file_piece_size (String.length !file_pieces); *)
+  let file_id = Sha1.string (Bencode.encode ~strict:false !file_info) in
+  let npieces = 1 + Int64.to_int ((!length -- one) // !file_piece_size) in
   let pieces = Array.init npieces (fun i ->
         let s = String.sub !file_pieces (i*20) 20 in
         Sha1.direct_of_string s
     ) in
 
-(*  if !file_files <> [] && not (String2.check_suffix !file_name ".torrent") then
-    file_name := !file_name ^ ".torrent";*)
-  file_files := List.rev !file_files;
+(* Only at this point we know if the torrent contains an "encoding" field
+   If UTF8 filenames were found, use them. If not and we have a charset
+   value used for encoding, convert non-UTF8 filenames to UTF8 ones. *)
+  if !file_files_utf8 <> [] then
+    file_files := !file_files_utf8
+  else
+    if !file_encoding <> "" then
+      begin
+        let file_files_encoded = ref [] in
+        List.iter (fun (name, length) ->
+          file_files_encoded := !file_files_encoded @ [(Charset.safe_convert !file_encoding name), length]
+        ) !file_files;
+        file_files := !file_files_encoded
+      end;
+
+  (match List.length !file_files with
+  | 0 -> ()
+  | 1 -> file_name := (fst (List.hd !file_files));
+         file_files := []
+  | _ -> file_files := List.rev !file_files);
 
   file_id, {
-    torrent_name = !file_name;
+    torrent_name = real_file_name;
     torrent_filename = !file_torrent_filename;
-    torrent_name_utf8 = !file_name_utf8;
+    torrent_name_utf8 = real_file_name;
     torrent_length = !length;
     torrent_announce = !announce;
     torrent_announce_list = !announce_list;
     torrent_piece_size = !file_piece_size;
     torrent_files = !file_files;
     torrent_pieces = pieces;
-    torrent_comment = !file_comment;
-    torrent_created_by = !file_created_by;
+    torrent_comment = Charset.safe_convert !file_encoding !file_comment;
+    torrent_created_by = Charset.safe_convert !file_encoding !file_created_by;
     torrent_creation_date = !file_creation_date;
-    torrent_modified_by = !file_modified_by;
+    torrent_modified_by = Charset.safe_convert !file_encoding !file_modified_by;
     torrent_encoding = !file_encoding;
     torrent_private = !file_is_private;
 
@@ -326,8 +341,8 @@ let encode_torrent torrent =
 
   let encode_file (filename, size) =
     Dictionary [
-      String "length", Int size;
-      String "path", List (List.map
+      "length", Int size;
+      "path", List (List.map
           (fun s -> String s)(Filepath.string_to_path '/' filename));
     ]
   in
@@ -335,20 +350,20 @@ let encode_torrent torrent =
   let files =
     match torrent.torrent_files with
       [] ->
-        String "length", Int torrent.torrent_length
+        "length", Int torrent.torrent_length
     | _ ->
-        String "files",
+        "files",
         List (List.map encode_file torrent.torrent_files)
   in
 
   let info =
     Dictionary [
       files;
-      String "name", String torrent.torrent_name;
-      String "name.utf-8", String torrent.torrent_name_utf8;
-      String "piece length", Int torrent.torrent_piece_size;
-      String "pieces", String pieces;
-      String "private", Int torrent.torrent_private;
+      "name", String torrent.torrent_name;
+      "name.utf-8", String torrent.torrent_name_utf8;
+      "piece length", Int torrent.torrent_piece_size;
+      "pieces", String pieces;
+      "private", Int (if torrent.torrent_private then 1L else 0L);
     ]
   in
 
@@ -356,13 +371,13 @@ let encode_torrent torrent =
   let file_id = Sha1.string info_encoded in
   file_id,
   Dictionary [
-    String "announce", String torrent.torrent_announce;
-    String "comment", String torrent.torrent_comment;
-    String "created by", String torrent.torrent_created_by;
-    String "creation date", Int torrent.torrent_creation_date;
-    String "encoding", String torrent.torrent_encoding;
-    String "info", info;
-    String "modified-by", String torrent.torrent_modified_by;
+    "announce", String torrent.torrent_announce;
+    "comment", String torrent.torrent_comment;
+    "created by", String torrent.torrent_created_by;
+    "creation date", Int torrent.torrent_creation_date;
+    "encoding", String torrent.torrent_encoding;
+    "info", info;
+    "modified-by", String torrent.torrent_modified_by;
 (*
     String "nodes", String torrent.torrent_nodes;
 *)
@@ -370,7 +385,7 @@ let encode_torrent torrent =
 
 let make_torrent announce filename comment is_private =
   let announce_list = [ announce ] in
-  let basename = Filename.basename filename in
+  let basename = Filename2.basename filename in
   let files, t =
     if Unix2.is_directory filename then
       let rec iter_directory list dirname =
@@ -417,7 +432,7 @@ let make_torrent announce filename comment is_private =
   {
     torrent_name = basename;
     torrent_filename = "";
-    torrent_name_utf8 = Charset.to_utf8 basename;
+    torrent_name_utf8 = Charset.Locale.to_utf8 basename;
     torrent_length = length;
     torrent_announce = announce;
     torrent_announce_list = announce_list;
@@ -444,4 +459,6 @@ let generate_torrent announce torrent_filename torrent_comment torrent_private f
   let torrent = make_torrent announce filename torrent_comment torrent_private in
   let file_id, encoded = encode_torrent torrent in
   let encoded = Bencode.encode encoded in
-  File.from_string torrent_filename encoded
+  File.from_string torrent_filename encoded;
+  file_id
+
