@@ -215,8 +215,8 @@ let _ =
   network.op_network_connected <- (fun _ ->
     !connected_servers <> []
   );
-  network.op_network_parse_url <- (fun _ _ -> empty_string, false );
-  network.op_network_download <- (fun r _ -> DcInteractive.start_result_download r );
+  network.op_network_parse_url <- (fun url user group -> DcInteractive.parse_url url user group);
+  network.op_network_download <- DcInteractive.start_result_download;
   network.op_network_ports <- (fun _ ->
     [
     !!dc_port, "client_port TCP+UDP";
@@ -241,6 +241,7 @@ let _ =
 (*  network.op_network_clean_exit <- (fun _ -> lprintf_nl "Received (op_network_clean_exit)"; ()); *)
   network.op_network_reset <- (fun _ -> ());
   network.op_network_porttest_result <- (fun _ -> PorttestNotAvailable);
+  network.op_network_check_upload_slots <- (fun _ -> ());
 
   CommonInteractive.register_gui_options_panel "DC" gui_dc_options_panel; 
   
