@@ -132,6 +132,13 @@ let reset_tags () =
     m.emule_secident <- secident;
     m.emule_features <- secident;
 
+  let advertise_browse =
+    match !!allow_browse_share with
+        1 | 2 -> 0
+      | _ -> 1
+  in
+   m.emule_noviewshared <- advertise_browse;
+  
   let emule_miscoptions1 = D.emule_miscoptions1 m in
   let emule_miscoptions2 = D.emule_miscoptions2 m in
   let emule_compatoptions = D.emule_compatoptions m in
@@ -263,12 +270,15 @@ let enable () =
 be useful when users want to share files that they had already previously
   shared *)
           let key = (file.sh_name, file.sh_size, file.sh_mtime) in
+          (try
           if Unix32.file_exists file.sh_name &&
             not (Hashtbl.mem shared_files_info key) 
             then begin
               Hashtbl.add shared_files_info key file;
               list := file :: !list
             end
+          with e ->
+            lprintf_nl "ignoring share: %s" (Printexc2.to_string e))
       ) !!known_shared_files;
       known_shared_files =:= !list;
 
@@ -278,7 +288,7 @@ be useful when users want to share files that they had already previously
           let sock = TcpServerSocket.create 
               "donkey client server"
               (Ip.to_inet_addr !!client_bind_addr)
-            !!donkey_port (client_connection_handler false) in
+            !!donkey_port ~backlog:!!max_upload_slots (client_connection_handler false) in
           
           TcpServerSocket.set_accept_controler sock connections_controler;
           listen_sock := Some sock;
@@ -317,6 +327,7 @@ be useful when users want to share files that they had already previously
       Options.option_hook global_login reset_tags;
       Options.option_hook login reset_tags;
       Options.option_hook enable_sui reset_tags;
+      Options.option_hook allow_browse_share reset_tags;
 				
 (**** START TIMERS ****)
       add_session_option_timer enabler check_client_connections_delay 
@@ -368,6 +379,11 @@ let rec update_options () =
       if !!upload_timeout = 60. then
         upload_timeout =:= 600.;
       update 3
+  | 3 ->
+      propagate_sources =:= false;
+      update_server_list_server =:= false;
+      upload_full_chunks =:= true;
+      update 4
   | _ -> ()
 
 
