@@ -31,8 +31,8 @@ let max_xs_packets = define_expert_option donkey_section ["max_xs_packets"]
 
 let donkey_port = define_option donkey_section ["port"]
   ~restart: true
-  "The port used for connection by other donkey clients."
-    int_option (2000 + Random.int 20000)
+  "The port used for TCP connection by other donkey clients. UDP port = port + 4."
+    port_option (2000 + Random.int 20000)
 
 let check_client_connections_delay = define_expert_option donkey_section ["check_client_connections_delay"]
   "Delay used to request file sources"
@@ -43,11 +43,11 @@ let client_timeout = define_expert_option donkey_section ["client_timeout"]
     float_option 40.
 
 let max_connected_servers = define_option donkey_section ["max_connected_servers"]
-  "The number of servers you want to stay connected to"
+  "The number of servers you want to stay connected to, maximum allowable = 3"
     int_option 3
 
 let max_allowed_connected_servers () =
-  min (int_of_string ((strings_of_option max_connected_servers).option_default))
+  min (int_of_string (strings_of_option max_connected_servers).option_default)
       !!max_connected_servers
 
 let reliable_sources = define_option donkey_section ["reliable_sources"]
@@ -63,7 +63,7 @@ let server_black_list = define_option donkey_section ["server_black_list"]
   Servers on this list can't be added, and will eventually be removed"
     CommonOptions.ip_range_list_option []
 
-let server_black_list_set = ref Ip_set.BL_Empty
+let server_black_list_set = ref Ip_set.bl_empty
 
 let () =
   option_hook server_black_list (fun _ ->
@@ -80,7 +80,7 @@ let force_client_high_id = define_option donkey_section ["force_client_high_id"]
 let update_server_list_server = define_option donkey_section ["update_server_list_server"]
   "Set this option to false if you don't want to
   receive new servers from servers"
-    bool_option true
+    bool_option false
 
 let update_server_list_server_met = define_option donkey_section ["update_server_list_server_met"]
   "Set this option to false if you don't want to
@@ -116,8 +116,10 @@ let log_clients_on_console = define_expert_option donkey_section ["log_clients_o
     bool_option false
 
 let propagate_sources = define_expert_option donkey_section ["propagate_sources"]
-  "Allow mldonkey to propagate your sources to other donkey clients"
-    bool_option true
+  "Allow mldonkey to propagate your sources to other mldonkey clients.
+   This function is superseded by eMule-style source exchange,
+   this option is outdated"
+    bool_option false
 
 let max_sources_per_file = define_option donkey_section ["max_sources_per_file"]
   "Maximal number of sources for each file"
@@ -177,7 +179,7 @@ let upload_lifetime = define_expert_option donkey_section ["upload_lifetime"]
 
 let upload_full_chunks = define_expert_option donkey_section ["upload_full_chunks"]
   "If true, each client is allowed to receive one chunk, this setting overrides upload_lifetime"
-    bool_option false
+    bool_option true
 
 let upload_complete_chunks = define_expert_option donkey_section ["upload_complete_chunks"]
   "If true, each client is allowed to complete only one chunk, independent, if it is empty or
@@ -193,56 +195,6 @@ let dynamic_upload_threshold = define_expert_option donkey_section ["dynamic_upl
   "Uploaded zones (1 zone = 180 kBytes) needed to enable the dynamic upload lifetime"
     int_option 10
 
-let upload_compression = define_expert_option donkey_section ["upload_compression"]
-  "Enables compressed upload as part of the protocol"
-    bool_option true
-
-let upload_compression_threshold = define_expert_option donkey_section ["upload_compression_threshold"]
-  "Sizedifference in bytes between one zone (180 kBytes) and its compressed
-  counterpart, which has to occure, to send compressed parts instead of plain."
-    int_option 2000
-
-let upload_compression_ext_exclude = define_expert_option donkey_section ["upload_compression_ext_exclude"]
-  "Disable upload compression based on file extensions (without dot)"
-    string_list_option ["zip"; "7z"; "gz"; "bz2"; "rar"; "ace"; "ogm"; "avi"; "mpg"]
-
-let _ =
-  option_hook upload_compression_threshold (fun _ ->
-    if !!upload_compression_threshold < 0 then
-        upload_compression_threshold =:= 0
-  );
-  option_hook upload_compression_ext_exclude (fun _ ->
-    let l = List.map String.lowercase !!upload_compression_ext_exclude in
-    if !!upload_compression_ext_exclude <> l then upload_compression_ext_exclude =:= l
-  )
-
-let upload_compression_level = define_expert_option donkey_section ["upload_compression_level"]
-  "Level of the used zlibcompression. allowed are values between 0 and 9. higher
-  level means better compression, but higher cpu usage too. (emules default
-  compression level for compressed parts is 9)"
-    int_option 9
-
-let _ =
-  option_hook upload_compression_level (fun _ ->
-    if !!upload_compression_level < 0
-      || !!upload_compression_level > 9 then
-        upload_compression_level =:= 9
-  )
-
-let upload_compression_table_size = define_expert_option donkey_section ["upload_compression_table_size"]
-  ~restart: true
-  "Size of the cache table in entries (ca. 2 * 180 kbytes). zones have to be
-  compressed at once, but only parts of it are sent at a time (10 kbytes).
-  to reduce diskaccess and repeated compression to a minimum, size should be
-  at least the number of total upload slots. restart of core is required."
-    int_option 20
-
-let _ =
-  option_hook upload_compression_table_size (fun _ ->
-    if !!upload_compression_table_size < 1 then
-        upload_compression_table_size =:= 1
-  )
-
 let connected_server_timeout = define_expert_option donkey_section ["connected_server_timeout"]
   "How long can a silent server stay connected"
     float_option 1800.
@@ -331,12 +283,12 @@ let overnet_section = file_section donkey_ini [ overnet_options_section_name ]
 let overnet_port = define_option overnet_section [overnet_options_section_name; "port"]
   ~restart: true
   "port for overnet"
-    int_option (2000 + Random.int 20000)
+    port_option (2000 + Random.int 20000)
 
 let options_version = define_expert_option donkey_section ["options_version"]
   ~internal: true
   "(internal option)"
-    int_option 3
+    int_option 4
 
 let gui_donkey_options_panel =
   [
