@@ -237,6 +237,7 @@ module P = struct
         peer_ip = ip;
         peer_port = udp_port;
         peer_tcpport = tcp_port;
+        peer_country_code = Geoip.get_country_code_option ip;
         peer_kind = 3;
         peer_last_send = 0;
         peer_expire = 0;
@@ -273,6 +274,7 @@ module P = struct
             peer_ip = !peer_ip;
             peer_port = !peer_udpport;
             peer_tcpport = !peer_tcpport;
+            peer_country_code = Geoip.get_country_code_option !peer_ip;
             peer_md4 = r_md4;
             peer_last_send = 0;
             peer_expire = 0;
@@ -419,7 +421,7 @@ module P = struct
       let opcode = int_of_char pbuf.[1] in
       let msg = String.sub pbuf 2 (len-2) in
       let msg = if magic = kademlia_packed_header_code then
-          let s = Autoconf.zlib__uncompress_string2 msg in
+          let s = Zlib.uncompress_string2 msg in
 (*          lprintf "Uncompressed:\n";
           dump s; *)
           s
@@ -438,7 +440,7 @@ module P = struct
           if String.length s > 200 then
             let opcode = String.sub s 0 1 in
             let args = String.sub s 1 (String.length s - 1) in
-            kademlia_packed_header ^ opcode ^ (Autoconf.zlib__compress_string args)
+            kademlia_packed_header ^ opcode ^ (Zlib.compress_string args)
           else
             kademlia_header ^ s
         in
@@ -476,7 +478,7 @@ module P = struct
                 in
                 let t = parse_message ip port pbuf in
                 let is_not_banned ip =
-                  match !Ip.banned ip with
+                  match !Ip.banned (ip, None) with
                     None -> true
                    | Some reason ->
                   if !verbose_overnet then
