@@ -80,8 +80,9 @@ let new_file_to_share sh codedname old_impl =
     if !verbose_share then
       lprintf_nl "Sharing file with MD4: %s" (Md4.to_string md4);
 
+    let user = CommonUserDb.admin_user () in
     let file = new_file sh.sh_name FileShared md4 sh.sh_size
-        "" false (CommonUserDb.admin_user ()) in
+        "" false user user.user_default_group in
     must_share_file file codedname old_impl;
     file.file_computed_md4s <- md4s;
     add_file_filenames (as_file file) (Filename.basename sh.sh_name);
