@@ -63,6 +63,7 @@ type emule_proto = {
     mutable emule_extmultipacket : int;
     mutable emule_largefiles : int;
     mutable emule_kad_version : int;
+    mutable emule_support_captcha : int;
   }
 
 type emule_tag_name =
@@ -460,6 +461,7 @@ and server = (*[]*){
     mutable server_cid : Ip.t option;
     mutable server_port : int;
     mutable server_realport : int option; (* in case we connect through auxport; this is the true one *)
+    mutable server_country_code : int option;
     mutable server_sock : tcp_connection;
     mutable server_search_queries : CommonTypes.search Fifo.t;
     mutable server_users_queries : bool Fifo.t;
@@ -552,6 +554,7 @@ and client = {
     client_source : DonkeySources.source;
     mutable client_md4 : Md4.t;
     mutable client_ip : Ip.t;
+    mutable client_country_code : int option;
     mutable client_download : (file * CommonSwarming.uploader) option;
     mutable client_file_queue : (
       file * (* has displayed when connected *)
@@ -728,6 +731,7 @@ let dummy_emule_proto = {
     emule_extmultipacket = 0;
     emule_largefiles = 0;
     emule_kad_version = 0;
+    emule_support_captcha = 0;
   }
 
 let emule_proto () =
