@@ -55,20 +55,8 @@ open FileTPClients
 let default_user = "anonymous"
 let default_pass = "-mldonkey@"
 
-let get_user url = 
-  let url_user = url.Url.user in
-  let user = if url_user <> "" 
-    then url_user 
-    else default_user
-  in user
-
-let get_pass url = 
-  let url_pass = url.Url.passwd in
-  let pass = if url_pass <> "" 
-    then "-" ^ url_pass 
-    else default_pass
-  in pass
-
+let get_user url = if url.Url.user = "" then default_user else url.Url.user
+let get_pass url = if url.Url.passwd = "" then default_pass else url.Url.passwd
 
 let reg530 = Str.regexp ".*not connect more.*\\|.*too many.*\\|.*overloaded.*\\|.*try \\(again \\|back \\)?later.*\\|.*is restricted to.*\\|.*maximum number.*\\|.*only.*session.*allowed.*\\|.*more connection.*"
 
@@ -197,6 +185,11 @@ let write_reqs sock reqs =
 (*                                                                       *)
 (*************************************************************************)
 
+let get_path_components s =
+  match List.rev (List.map (Url.decode ~raw:true) (String2.split s '/')) with
+  | file::path -> String.concat "/" (List.rev path), file
+  | [] -> ("","") (* empty path *)
+
 let ftp_send_range_request c (x,y) sock d =
 
   if !verbose then lprintf_nl "Asking range %Ld-%Ld" x y;
@@ -242,8 +235,14 @@ let ftp_send_range_request c (x,y) sock d =
                   let reqs = ["TYPE I"] in
                   write_reqs sock reqs;
                 | "200 " -> 
-                  let reqs = [Printf.sprintf "CWD %s" (Filename.dirname file)] in
-                  write_reqs sock reqs; 
+                  let reqs =
+                    match get_path_components file with
+                    | ("",_) -> (* no CWD needed *) ["PASV"]
+                    | (dir,_) -> [Printf.sprintf "CWD %s" dir]
+                  in
+                  (* FIXME should really issue several CWDs (one for each directory)
+                     TODO implement state for protocol *)
+                  write_reqs sock reqs;
                 | "227 " ->
                   (try
                   let pos = String.index line '(' in
@@ -264,8 +263,8 @@ let ftp_send_range_request c (x,y) sock d =
                 with e ->
                     lprintf_nl "Error %s in reader" (Printexc.to_string e);
                     close sock Closed_by_user)
-                | "350 " ->  
-                    let reqs = [Printf.sprintf "RETR %s" (Filename.basename file)] in
+                | "350 " ->
+                    let reqs = [Printf.sprintf "RETR %s" (snd (get_path_components file))] in
                     write_reqs sock reqs; 
                 | "150 " ->
                     if !verbose then begin
@@ -279,13 +278,13 @@ let ftp_send_range_request c (x,y) sock d =
                 | "530 " ->
                     let reason = String.sub line 4 (slen - 4) in
                     if not (retry_530 reason) then begin
-                      pause_for_cause d.download_file "530" CommonUserDb.admin_user;
+                      pause_for_cause d.download_file "530" (CommonUserDb.admin_user ());
                     end else begin
                       c.client_reconnect <- true;
                     end;
                     disconnect_client c Closed_by_user;
                 | "550 " ->
-                    pause_for_cause d.download_file "550" CommonUserDb.admin_user;
+                    pause_for_cause d.download_file "550" (CommonUserDb.admin_user ());
                     disconnect_client c Closed_by_user;
                 | _ -> 
                     if !verbose then lprintf_nl "Unexpected line %s" line;
@@ -384,8 +383,7 @@ let ftp_check_size file url start_download_file =
 
 *)
 
-  let dirname =  Filename.dirname url.Url.full_file in
-  let basename = Filename.basename url.Url.full_file in
+  let (dirname,basename) = get_path_components url.Url.full_file in
   let server, port = url.Url.server, url.Url.port in
 (*    lprintf "async_ip ...\n"; *)
   Ip.async_ip server (fun ip ->
@@ -429,7 +427,11 @@ let ftp_check_size file url start_download_file =
                       let reqs = ["TYPE I"] in
                       write_reqs sock reqs;
                     | "200 " ->
-                      let reqs = [Printf.sprintf "CWD %s" dirname] in
+                      let reqs =
+                        match dirname with
+                        | "" -> (* no CWD needed *) [Printf.sprintf "SIZE %s" basename]
+                        | dir -> [Printf.sprintf "CWD %s" dir]
+                      in
                       write_reqs sock reqs;
                     | "250 " -> 
                       let reqs = [Printf.sprintf "SIZE %s" basename] in
@@ -444,11 +446,11 @@ let ftp_check_size file url start_download_file =
                     | "530 " ->
                       let reason = String.sub line 4 (slen - 4) in
                       if not (retry_530 reason) then begin
-                        pause_for_cause file "530" CommonUserDb.admin_user;
+                        pause_for_cause file "530" (CommonUserDb.admin_user ());
                       end;
                       close sock Closed_by_user;
                     | "550 " ->
-                      pause_for_cause file "550" CommonUserDb.admin_user;
+                      pause_for_cause file "550" (CommonUserDb.admin_user ());
                       close sock Closed_by_user;
                     | _ -> 
                       if !verbose then lprintf_nl "Unexpected line %s" line;
