@@ -97,6 +97,10 @@ let _ =
         | None -> None 
 	| Some swarmer ->
             Some (CommonSwarming.chunks_verified_bitmap swarmer));
+        P.file_chunk_size = (match file.file_swarmer with
+        | None -> None 
+	| Some t ->
+            Some (List.map (fun t -> t.CommonSwarming.t_chunk_size) t.CommonSwarming.t_s.CommonSwarming.s_networks));
         P.file_availability =
         [network.network_num,(match file.file_swarmer with
           None -> "" | Some swarmer ->
@@ -112,10 +116,12 @@ let string_of_client_addr c = c.client_hostname
 
 let _ =
   client_ops.op_client_info <- (fun c ->
+      check_client_country_code c;
       { (impl_client_info c.client_client) with
         P.client_network = network.network_num;
         P.client_kind = Known_location (Ip.from_name c.client_hostname,
           c.client_port);
+        P.client_country_code = c.client_country_code;
         P.client_state = client_state (as_client c);
         P.client_type = client_type c;
         P.client_name = (Printf.sprintf "%s:%d"
@@ -156,7 +162,9 @@ let _ =
 	    Indirect_location (_, _, ip, port)
 	  | Known_location (ip, port) -> ip, port
 	in
-        let ccode,cname = Geoip.get_country (fst (client_ip,client_port)) in
+        let ccode,cname =
+          Geoip.get_country_code_name cinfo.GuiTypes.client_country_code
+        in
 
         Printf.bprintf buf " \\<tr onMouseOver=\\\"mOvr(this);\\\"
     onMouseOut=\\\"mOut(this);\\\" class=\\\"%s\\\"\\>" str;
@@ -183,7 +191,7 @@ let _ =
           ("", "sr", "D");
           ("", "sr", "N");
           ("", "sr", Printf.sprintf "%s:%d" (Ip.to_string client_ip) client_port);
-          ] @ (if !Geoip.active then [(cname, "sr", CommonPictures.flag_html ccode)] else []) @ [
+          ] @ (if Geoip.active () then [(cname, "sr", CommonPictures.flag_html ccode)] else []) @ [
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_uploaded));
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_total_downloaded));
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_session_uploaded));
@@ -237,7 +245,7 @@ and find_mirrors file u =
 
 let previous_url = ref ""  
   
-let download_file url referer user =
+let download_file url referer user group =
   let u = Url.of_string url in
 
   if List.mem u !!old_files && !previous_url <> url then begin
@@ -245,7 +253,7 @@ let download_file url referer user =
       failwith "URL already downloaded: repeat command again to force";
     end;
 
-  let file = new_file (Md4.random ()) u.Url.full_file zero user in
+  let file = new_file (Md4.random ()) u.Url.full_file zero user group in
   
   if !verbose then
     lprintf_nl "Started new download: %s from %s" (file_best_name file) url; 
@@ -268,7 +276,7 @@ let get_regexp_int text r =
   ignore (Str.search_forward r text 0);
   let a = Str.group_beginning 1 in
   let b = Str.group_end 1 in
-  int_of_string (String.sub text a (b - a))
+  Int64.of_string (String.sub text a (b - a))
 
 let get_regexp_string text r =
   ignore (Str.search_forward r text 0);
@@ -280,7 +288,7 @@ let get_regexp_string text r =
    It returns true if this file can be handled by fileTP,
    and false otherwise.
  *)
-let op_network_parse_url url user =
+let op_network_parse_url url user group =
   let location_regexp = "Location: \\(.*\\)" in
   let real_url = get_regexp_string url (Str.regexp location_regexp) in
   if (is_http_torrent url real_url) && !!enable_bittorrent then
@@ -289,8 +297,8 @@ let op_network_parse_url url user =
     if (String2.check_prefix real_url "http://") then (
       let length_regexp = "Content-Length: \\(.*\\)" in
        try let length = get_regexp_int url (Str.regexp length_regexp) in
-         if (length > 0) then begin
-           download_file real_url "" user; "started FileTP download", true
+         if (length > 0L) then begin
+           download_file real_url "" user group; "started FileTP download", true
          end
          else "can not parse Content-Length", false
        with Not_found -> 
@@ -299,7 +307,7 @@ let op_network_parse_url url user =
     else 
       if (String2.check_prefix real_url "ftp://") ||
          (String2.check_prefix real_url "ssh://") then (
-      download_file url "" user;
+      download_file real_url "" user group;
       "started FileTP download", true)
     else
       "invalid URL", false
@@ -314,8 +322,8 @@ let commands = [
     "http", "Network/FileTP", Arg_multiple (fun args o ->
         try
         (match args with
-          url :: [referer] -> download_file url referer o.conn_user.ui_user
-        | [url] -> download_file url "" o.conn_user.ui_user
+          url :: [referer] -> download_file url referer o.conn_user.ui_user o.conn_user.ui_user.user_default_group
+        | [url] -> download_file url "" o.conn_user.ui_user o.conn_user.ui_user.user_default_group
         | _ -> raise Not_found);
         let buf = o.conn_buf in
         if o.conn_output = HTML then
@@ -409,7 +417,7 @@ let _ =
   CommonNetwork.register_commands commands;
   network.op_network_share <- (fun fullname codedname size -> ());
   network.op_network_search <- (fun ss buf -> ());
-  network.op_network_download <- (fun r user -> dummy_file);
+  network.op_network_download <- (fun r user group -> dummy_file);
   file_ops.op_file_commit <- (fun file new_name -> clean_stop file);
   file_ops.op_file_pause <- (fun file -> 
     List.iter (fun c ->
@@ -419,6 +427,17 @@ let _ =
       | _ -> ()
     ) file.file_clients
   );
+  file_ops.op_file_download_order <- (fun file strategy ->
+      match file.file_swarmer with
+      | None -> None
+      | Some s ->
+          (match strategy with
+          (* return current strategy *)
+          | None -> Some (CommonSwarming.get_strategy s)
+          | Some strategy -> CommonSwarming.set_strategy s strategy;
+                      Some (CommonSwarming.get_strategy s))
+  );
+  file_ops.op_file_queue <- file_ops.op_file_pause;
   file_ops.op_file_resume <- (fun file -> ());
   file_ops.op_file_print <- (fun file buf -> ());
   network.op_network_close_search <- (fun s -> ());
@@ -426,4 +445,5 @@ let _ =
   network.op_network_connect_servers <- (fun s -> ());
   network.op_network_reset <- (fun _ -> ());
   network.op_network_porttest_result <- (fun _ -> PorttestNotAvailable);
+  network.op_network_check_upload_slots <- (fun _ -> ());
   network.op_network_recover_temp <- (fun s -> ())
