@@ -55,9 +55,14 @@ let range_arg = define_option fileTP_section ["range_arg"]
     string_option "range"
 
 let options_version = define_option fileTP_section ["options_version"]
+    ~internal: true
     "(internal option)"
     int_option 0
 
+let chunk_size = define_option fileTP_section ["chunk_size"]
+  "Chunk size (in bytes) (0 = No chunks)"
+  int_option 0
+
   (*
 let verbose_clients =
   define_option fileTP_section ["verbose_clients"]
