@@ -43,22 +43,37 @@ open GnutellaProto
 let redirectors_urlfiles = ref []
 let redirectors_hostfiles = ref []
   
-let parse_urlfile file = 
+let parse_urlfile file url_string = 
   let s = File.to_string file in
+  (* Http_client.wget does not delete the temp file anymore *)
+  (try Sys.remove file with _ -> ());
+  if !verbose then lprintf_nl "Parsing urlfile from %s:\n%s" url_string s;
   clean_file s;
   let lines = String2.split_simplify s '\n' in
   List.iter (fun line ->
-      if not (List.mem line !!gnutella_hostfiles) then
-        gnutella_hostfiles =:= line :: !!gnutella_hostfiles
+      if not (List.mem line !!gnutella_hostfiles) then begin
+        gnutella_hostfiles =:= line :: !!gnutella_hostfiles;
+        if !verbose then lprintf_nl "Added GWebCache %s" line;
+      end
   ) lines;
   redirectors_hostfiles := !!gnutella_hostfiles
 
+let next_urlfile_access = ref 0
+
 let connect_urlfile () = 
   match !redirectors_urlfiles with
     [] ->
-      redirectors_urlfiles := !!urlfiles
+      if !next_urlfile_access < last_time () then begin
+          (* 12 hour interval between urlfile re-connection attempts *)
+          next_urlfile_access := last_time () + (3600*12);
+          redirectors_urlfiles := !!urlfiles;
+          lprintf_nl "added %d urlfiles" (List.length !!urlfiles);
+      end else begin
+        if !verbose then lprintf_nl "connect_urlfile: no urlfiles, too soon";
+      end
   | url :: tail ->
       redirectors_urlfiles := tail;
+      let url_string = url in
       let module H = Http_client in
       let url = Printf.sprintf "%s?urlfile=1&client=MLDK&version=%s"
           url Autoconf.current_version in
@@ -66,25 +81,35 @@ let connect_urlfile () =
           H.basic_request with
           H.req_url = Url.of_string url;
           H.req_proxy = !CommonOptions.http_proxy;
-          H.req_user_agent = 
-          Printf.sprintf "MLDonkey/%s" Autoconf.current_version;
+          H.req_user_agent = get_user_agent ();
         } in
       if !verbose then
-        lprintf "Connecting Gnutella %s\n" url;
-      H.wget r parse_urlfile    
+        lprintf_nl "Connecting to urlfile %s\n" url;
+      H.wget r (fun filename -> parse_urlfile filename url_string)
       
-let parse_hostfile file = 
+let parse_hostfile file url_string = 
   let s = File.to_string file in
-  clean_file s;
-  let lines = String2.split_simplify s '\n' in
-  List.iter (fun line ->
+  if String2.starts_with s "ERROR" || String2.starts_with s "<" then begin
+    if !verbose then lprintf_nl "Malformed response content:\n%s" s;
+    if List.mem url_string !!gnutella_hostfiles then begin
+      gnutella_hostfiles =:= List.filter (fun h -> h <> url_string) !!gnutella_hostfiles;
+      redirectors_hostfiles := !!gnutella_hostfiles;
+      if !verbose then lprintf_nl "Removing %s from hostfiles" url_string;
+    end;
+  end
+  else begin
+    clean_file s;
+    let lines = String2.split_simplify s '\n' in
+    if !verbose then lprintf_nl "Parsing response from %s:\n%s" url_string s;
+    List.iter (fun line ->
       try
         let ip, port = String2.cut_at line ':' in
         if !verbose then
-          lprintf "gnutella1: adding ultrapeer from hostfile\n";
+          lprintf_nl "Adding ultrapeer from hostfile %s %s" ip port;
         ignore (H.new_host (Ip.addr_of_string ip) (int_of_string port) Ultrapeer)
       with _ -> ()
-  ) lines
+    ) lines
+  end
 
 let next_redirector_access = ref 0
   
@@ -92,29 +117,29 @@ let connect_hostfile _ =
   match !redirectors_hostfiles with
     [] ->
       if !next_redirector_access < last_time () then begin
-          (* We should only contact the redirectors
-             if we don't have enough hosts.
-             Changed it to once every day,
-             so we don't hurt the network. *)
-          next_redirector_access := last_time () + (3600*24);
-          connect_urlfile ();
+          (* 12 hour interval between redirector re-connection attempts *)
+          next_redirector_access := last_time () + (3600*12);
           redirectors_hostfiles := !!gnutella_hostfiles
-        end;
+      end 
+      else begin
+        if !verbose then lprintf_nl "connect_hostfile: no gwebcaches, too soon";
+      end;
+      connect_urlfile ();
   | url :: tail ->
       redirectors_hostfiles := tail;
       let module H = Http_client in
+      let url_string = url in
       let url = Printf.sprintf "%s?hostfile=1&client=MLDK&version=%s"
           url Autoconf.current_version in
       let r = {
           H.basic_request with
           H.req_url = Url.of_string url;
           H.req_proxy = !CommonOptions.http_proxy;
-          H.req_user_agent = 
-          Printf.sprintf "MLDonkey/%s" Autoconf.current_version;
+          H.req_user_agent = get_user_agent ();
         } in
       if !verbose then
-        lprintf "Connecting Gnutella %s\n" url;
-      H.wget r parse_hostfile    
+        lprintf_nl "Connecting to hostfile %s" url;
+      H.wget r (fun filename -> parse_hostfile filename url_string)
       
 let connect _ = 
   connect_hostfile ()
