@@ -172,7 +172,7 @@ external is_utf8 : string -> bool = "ml_utf8_validate"
 (**********************************************************************************)
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.9 2006/10/08 14:12:13 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.12 2010/05/24 18:10:49 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 let utf8_look s i =
@@ -243,9 +243,9 @@ let rec nth_aux s i n =
   if n = 0 then i else
   nth_aux s (utf8_next s i) (n - 1)
 
-let nth s n = nth_aux s 0 n
+let utf8_nth s n = nth_aux s 0 n
 
-let utf8_get s n = utf8_look s (nth s n)
+let utf8_get s n = utf8_look s (utf8_nth s n)
 
 (**********************************************************************************)
 (*                                                                                *)
@@ -254,7 +254,7 @@ let utf8_get s n = utf8_look s (nth s n)
 (**********************************************************************************)
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.9 2006/10/08 14:12:13 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.12 2010/05/24 18:10:49 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 let rec length_aux s c i =
@@ -281,7 +281,7 @@ let utf8_length s = length_aux s 0 0
 
 
 (* taken from camomile *)
-(* $Id: charset.ml,v 1.9 2006/10/08 14:12:13 spiralvoice Exp $ *)
+(* $Id: charset.ml,v 1.12 2010/05/24 18:10:49 spiralvoice Exp $ *)
 (* Copyright 2002, 2003 Yamagata Yoriyuki. distributed with LGPL *)
 
 external uint_code : uchar -> int = "%identity"
@@ -1646,21 +1646,42 @@ let western_european =
    [I_863; CP863; IBM863; CSIBM863];
   ]
 
+let convert ~from_charset ~to_charset s =
+  if s <> "" then begin
+    let t = charset_to_string to_charset in
+    let f = charset_to_string from_charset in
+    convert_string s t f
+  end else s
+
+let safe_convert enc s =
+  match enc with
+  | "" -> s
+  | enc ->
+    try
+      convert
+        ~from_charset: (charset_from_string enc)
+        ~to_charset:UTF_8
+        s
+    with _ -> s
+
+(* Locale specific conversions *)
+module Locale = struct
+
+(* FIXME move away! *)
+let () =
+  (* block signals until core started correctly *)
+  (MlUnix.set_signal  Sys.sigint
+    (Sys.Signal_handle (fun _ -> ())));
+  (MlUnix.set_signal  Sys.sigterm
+    (Sys.Signal_handle (fun _ -> ())))
+
 let locale =
   try
     let cs = get_charset () in
     charset_from_string cs
   with _ -> ASCII
 
-let locstr =
-  (* block signals until core started correctly *)
-  (MlUnix.set_signal  Sys.sigint
-    (Sys.Signal_handle (fun _ -> ())));
-  (MlUnix.set_signal  Sys.sigterm
-    (Sys.Signal_handle (fun _ -> ())));
-
-  let s = charset_to_string locale in
-  s
+let locale_string = charset_to_string locale
 
 let (enc_list : string list ref) = ref []
 let nenc = ref 0
@@ -1699,12 +1720,6 @@ let all_regions =
   ]
 *)
 
-(**********************************************************************************)
-(*                                                                                *)
-(*                          charset_list_from_language                            *)
-(*                                                                                *)
-(**********************************************************************************)
-
 (* See http://www.gnu.org/software/gettext/manual/html_chapter/gettext_15.html#SEC221
  * The strategy is not perfect. Any comment to improve it, is highly appreciated.
  * The charset list shall be improved according to the language detected on the
@@ -1788,12 +1803,6 @@ let charset_list_from_language lang =
   in
   List.flatten !li
 
-(**********************************************************************************)
-(*                                                                                *)
-(*                          set_default_charset_list                              *)
-(*                                                                                *)
-(**********************************************************************************)
-
 let set_default_charset_list (lang : string) =
   (* Let's get rid of charset aliases *)
   let l = List.map (fun li -> List.hd li) (charset_list_from_language lang) in
@@ -1804,38 +1813,8 @@ let set_default_charset_list (lang : string) =
   ) !enc_list; *)
   nenc := List.length !enc_list
 
-(**********************************************************************************)
-(*                                                                                *)
-(*                          convert                                               *)
-(*                                                                                *)
-(**********************************************************************************)
-
 let conversion_enabled = ref true
 
-let convert ~from_charset ~to_charset s =
-  if s <> "" then begin
-    let t = charset_to_string to_charset in
-    let f = charset_to_string from_charset in
-    convert_string s t f
-  end else s
-
-let safe_convert enc s =
-  match enc with
-    "" -> s
-  | enc ->
-    try
-      convert
-        ~from_charset: (charset_from_string enc)
-        ~to_charset: (charset_from_string "UTF-8")
-        s
-    with _ -> s
-
-(**********************************************************************************)
-(*                                                                                *)
-(*                          slow_encode_from_utf8                                 *)
-(*                                                                                *)
-(**********************************************************************************)
-
 let slow_encode_from_utf8 s to_codeset =
   let us = ref "" in
   let slen = utf8_length s in
@@ -1853,12 +1832,6 @@ let slow_encode_from_utf8 s to_codeset =
   done;
   !us
 
-(**********************************************************************************)
-(*                                                                                *)
-(*                          slow_encode                                           *)
-(*                                                                                *)
-(**********************************************************************************)
-
 let slow_encode s to_codeset =
   if is_utf8 s
   then slow_encode_from_utf8 s to_codeset
@@ -1867,19 +1840,13 @@ let slow_encode s to_codeset =
     let slen = String.length s in
     for i = 0 to (slen - 1) do
       try
-        us := !us ^ (convert_string (String.sub s i 1) to_codeset locstr)
+        us := !us ^ (convert_string (String.sub s i 1) to_codeset locale_string)
       with _ ->
         us := !us ^ char_const
     done;
     !us
   end
 
-(**********************************************************************************)
-(*                                                                                *)
-(*                          fast_encode                                           *)
-(*                                                                                *)
-(**********************************************************************************)
-
 let fast_encode s to_codeset =
   let rec iter i n =
      if i = n 
@@ -1891,12 +1858,6 @@ let fast_encode s to_codeset =
          with _ -> iter (i + 1) !nenc
   in iter 0 !nenc
 
-(**********************************************************************************)
-(*                                                                                *)
-(*                          to_utf8                                               *)
-(*                                                                                *)
-(**********************************************************************************)
-
 let to_utf8 s =
   if s = ""
   then s
@@ -1904,12 +1865,6 @@ let to_utf8 s =
   then s
   else fast_encode s "UTF-8"
 
-(**********************************************************************************)
-(*                                                                                *)
-(*                          to_locale                                             *)
-(*                                                                                *)
-(**********************************************************************************)
-
 let to_locale s =
   if s = "" || not !conversion_enabled
   then s
@@ -1920,11 +1875,14 @@ let to_locale s =
       | _ ->
 	  begin
             try
-              convert_string s locstr "UTF-8"
+              convert_string s locale_string "UTF-8"
             with _ ->
-              slow_encode_from_utf8 s locstr
+              slow_encode_from_utf8 s locale_string
           end
   end
 
-let _ =
+let () =
   set_default_charset_list default_language
+
+end (* Locale *)
+
