@@ -49,6 +49,10 @@ let bin2dec num =
   let s = string_of_int num in
     int_of_string ("0b" ^ s)
 
+let percentage_of_ints v percent =
+  int_of_float (
+  (float_of_int v *. float_of_int percent /. 100.0) +. 0.5)
+
 let zip_extract_entry ifile e =
   if e.Zip.is_directory then begin
     try
@@ -63,12 +67,13 @@ let zip_extract zipfile =
     List.iter (zip_extract_entry ic) (Zip.entries ic))
 
 let rec zip_add_entry oc file =
-  let s = Unix.stat file in
-  match s.Unix.st_kind with
+  let module U = Unix.LargeFile in
+  let s = U.stat file in
+  match s.U.st_kind with
     Unix.S_REG ->
-      Zip.copy_file_to_entry file oc ~mtime:s.Unix.st_mtime file
+      Zip.copy_file_to_entry file oc ~mtime:s.U.st_mtime file
   | Unix.S_DIR ->
-      Zip.add_entry "" oc ~mtime:s.Unix.st_mtime
+      Zip.add_entry "" oc ~mtime:s.U.st_mtime
         (if Filename.check_suffix file "/" then file else file ^ "/");
       Unix2.tryopen_dir file (fun d ->
 	try
@@ -88,7 +93,7 @@ let gz_extract filename =
   let file = ref "" in
   try
     let buffer = String.create 4096 in
-    let file_out = Filename.temp_file "arch_" ".tmp" in
+    let file_out = Filename2.temp_file "arch_" ".tmp" in
     file := file_out;
     Unix2.tryopen_read_gzip filename (fun ic ->
       Unix2.tryopen_write_bin file_out (fun oc ->
