@@ -31,6 +31,11 @@
 #include <sys/resource.h>
 #endif /* HAVE_SYS_RESOURCE_H */
 
+/* For proper FreeBSD version identification */
+#if defined(HAVE_SYS_PARAM_H)
+#include <sys/param.h>
+#endif
+
 #define lseek XXXXXXXXX
 #define read XXXXXXXXX
 #define ftruncate XXXXXXXXX
@@ -675,7 +680,7 @@ value ml_setlcnumeric(value no)
 
 #define NETDB_BUFFER_SIZE 10000
 
-#ifdef _WIN32
+#if defined(_WIN32) || ( defined(__FreeBSD_version) && ( ((__FreeBSD_version >= 504102) && (__FreeBSD_version < 600000)) || (__FreeBSD_version >= 600029) ) )
 #define GETHOSTBYADDR_IS_REENTRANT 1
 #define GETHOSTBYNAME_IS_REENTRANT 1
 #endif /* _WIN32 */
